"""

 MIT License

 Copyright (c) 2020 Benjamin Collins (kion @ dashgl.com)

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in all
 copies or substantial portions of the Software.
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 SOFTWARE.

"""

import math
import struct
from Mat4 import Mat4 

class NinjaMotion:

	def __init__(self):
		self.name = "";
		self.skeleton = []
		self.frames = 0
		self.fps = 30
		self.keyFrames = []
		return None

	def setName(self, name):
		self.name = name
		return None
	
	def setSkeleton(self, bones):
		self.skeleton = bones
		for i in range(len(bones)):
			self.keyFrames.append([])
		return None

	def setFrames(self, frames):
		self.frames = frames
		return None

	def appendKeyFrame(self, boneIndex, frame, vec3, type):
		keyFrames = self.keyFrames[boneIndex]

		for keys in keyFrames:
			if keys['frame'] != frame:
				continue
			keys[type] = vec3
			return None
		
		keyFrame = {
			'frame' : frame,
			type : vec3
		}

		keyFrames.append(keyFrame)
		return None
	
	def save(self, boneIndex):
		
		bone = self.skeleton[boneIndex]
		keyFrames = self.keyFrames[boneIndex]
		
		if len(keyFrames) == 0:
			keyFrames.append({
				'frame' : 0
			})
			keyFrames.append({
				'frame' : self.frames - 1
			})

		if 'scl' not in keyFrames[0].keys():
			keyFrames[0]['scl'] = bone.getScale()
		
		if 'pos' not in keyFrames[0].keys():
			keyFrames[0]['pos'] = bone.getPosition()
		
		if 'rot' not in keyFrames[0].keys():
			keyFrames[0]['rot'] = bone.getRotation()

		lastIndex = len(keyFrames) - 1

		if 'scl' not in keyFrames[lastIndex].keys():
			keyFrames[lastIndex]['scl'] = bone.getScale()
		
		if 'pos' not in keyFrames[lastIndex].keys():
			keyFrames[lastIndex]['pos'] = bone.getPosition()
		
		if 'rot' not in keyFrames[lastIndex].keys():
			keyFrames[lastIndex]['rot'] = bone.getRotation()

		for keyFrame in keyFrames:

			if 'rot' not in keyFrame.keys():
				continue

			mat4 = Mat4()
			mat4.rotate(keyFrame['rot'])
			keyFrame['quat'] = mat4.toQuat()
			print(keyFrame['quat'])


		return None
		
